/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: MIT-0.
 */

/**
 * A negative test for --div-by-zero-check flag
 */
void div_by_zero_check_harness() {
    int num, den;
    int div = num / den;
}
